// filename *************RxFifob.c********
// Pointer implementation of the receive FIFO
// Jonathan W. Valvano 2/8/05

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing",
//         Brooks-Cole, copyright (c) 2000,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002

// Copyright 2004 by Jonathan W. Valvano, valvano@mail.utexas.edu 
//    You may use, edit, run or distribute this file 
//    as long as the above copyright notice remains 

#include "RxFifob.h"
/* Number of characters in the Fifo
     the FIFO is full when it has FifoSize-1 characters */
unsigned char static volatile *RxPutPtb;    /* Pointer of where to put next */
unsigned char static volatile *RxGetPtb;    /* Pointer of where to get next */
                  /* FIFO is empty if PutPt=GetPt */
                  /* FIFO is full  if PutPt+1=GetPt */
char unsigned static RxFifob[RXFIFOSIZEb];     /* The statically allocated fifo data */
 
/*-----------------------RxFifob_Init----------------------------
  Initialize fifo to be empty
  Inputs: none
  Outputs: none */
void RxFifob_Init(void){unsigned char SaveCCR;
/* save previous interrupt enable */
asm  tpa  
asm  staa SaveCCR 
asm  sei          /* make atomic */
  RxPutPtb=RxGetPtb=&RxFifob[0];   /* Empty when PutPt=GetPt */
asm  ldaa SaveCCR
asm  tap          /* end critical section */
}

/*-----------------------RxFifob_Put----------------------------
  Enter one character into the fifo
  Inputs: 8-bit data
  Outputs: true if data is properly saved
  Since this is called by interrupt handlers no sei,cli*/
int RxFifob_Put(unsigned char data){
unsigned char volatile *tempPt;
  tempPt = RxPutPtb;
  *(tempPt++) = data;              /* try to Put data into fifo */
  if(tempPt==&RxFifob[RXFIFOSIZEb]){ /* need to wrap?*/
    tempPt = &RxFifob[0];
  }
  if(tempPt == RxGetPtb ){
    return(0);             /* Failed, fifo was full */
  }     
  else{
    RxPutPtb = tempPt;      /* Success, so update pointer */
    return(1);
  }
}

/*-----------------------RxFifob_Get----------------------------
  Remove one character from the fifo
  Inputs: pointer to place to save 8-bit data
  Outputs: true if data is valid */
int RxFifob_Get(unsigned char *datapt){ 
  if(RxPutPtb == RxGetPtb ){
    return(0);     /* Empty if PutPt=GetPt */
  }
  else{
    *datapt = *(RxGetPtb++);
    if(RxGetPtb==&RxFifob[RXFIFOSIZEb]){ 
       RxGetPtb = &RxFifob[0];
    }
    return(1); 
  }
}

/*-----------------------RxFifbo_Status----------------------------
  Check the status of the fifo
  Inputs: none
  Outputs: true if there is any data in the fifo */
int RxFifob_Status(void){ 
  return (RxPutPtb != RxGetPtb);
}
